declare type ExplicitAny = any;
export declare type LogLevel = 'trace' | 'debug' | 'info' | 'warn' | 'error';
export declare const ENV_LOG_LEVEL = "LOG_LEVEL";
export declare const ENV_LOG_TRACE_KEYS = "LOG_TRACE_KEYS";
export declare function getDefaultLogLevel(): LogLevel;
export declare function getDefaultTraceMap(): string[];
export declare function makeShouldLog(logLevel: LogLevel): (level: LogLevel) => boolean;
export interface LogTransportOptions {
    type: string;
}
export interface Transport {
    log(item: any): void;
}
export declare type TransportMaker = (options: LogTransportOptions) => Transport;
declare class TransportRegistry {
    readonly inner: {
        [key: string]: TransportMaker;
    };
    constructor();
    register(key: string, maker: TransportMaker): void;
    make(options: LogTransportOptions): Transport;
}
export declare const transports: TransportRegistry;
interface LogData {
    [key: string]: ExplicitAny;
}
export interface ILogService {
    trace(args: LogData): void;
    debug(args: LogData): void;
    info(args: LogData): void;
    warn(args: LogData): void;
    error(args: LogData): void;
    pushScope(scope: string, data?: LogData, startOver?: boolean): ILogService;
}
export declare class ScopedLogService implements ILogService {
    readonly inner: ILogService;
    readonly scope: string;
    readonly data: LogData;
    constructor(inner: ILogService, scope: string, data?: LogData);
    private _getScope;
    pushScope(scope: string, data?: LogData): ScopedLogService;
    trace(args: LogData): void;
    debug(args: LogData): void;
    info(args: LogData): void;
    warn(args: LogData): void;
    error(args: LogData): void;
    toJSON(): {
        $class: string;
    };
}
export interface LogServiceOptions {
    logLevel?: LogLevel;
    scope?: string;
    data?: LogData;
    startTime?: [number, number];
    traceKeys?: string[];
    transports?: Transport[];
}
export declare class LogService implements ILogService {
    readonly scope: string;
    readonly logLevel: LogLevel;
    readonly startTime: [number, number];
    readonly shouldLog: (logLevel: LogLevel) => boolean;
    readonly data: LogData;
    readonly transports: Transport[];
    constructor(options?: LogServiceOptions);
    private _genScope;
    pushScope(scope: string, data?: LogData, startOver?: boolean): LogService;
    trace(args: LogData): void;
    debug(args: LogData): void;
    info(args: LogData): void;
    warn(args: LogData): void;
    error(args: LogData): void;
    traceObject<T>(obj: T): T;
    private _log;
    private _reallyLog;
    private _normalize;
    toJSON(): {
        $class: string;
    };
}
interface LoggerConstructor<T> {
    new (...args: ExplicitAny[]): T;
}
export declare function tracedClass<T extends LoggerConstructor<ExplicitAny>>(ctor: T): T;
export {};
