"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var Promise = require("bluebird");
var ts = require("typescript");
var tsConfig = require("./tsconfig");
var finder = require("./tsconfig-finder");
var watcher = require("./js-watcher");
var BaseTscRunner = /** @class */ (function () {
    function BaseTscRunner(options) {
        this.config = options.config;
        this.logger = options.logger;
        this.finder = new finder.TsConfigFinder({ config: options.config });
        this._reportDiagnostic = this._reportDiagnostic.bind(this);
    }
    BaseTscRunner.prototype._reportSwitch = function (diagnostic, report) {
        switch (diagnostic.category) {
            case ts.DiagnosticCategory.Error:
                this.logger.error(__assign({ code: "TS" + diagnostic.code }, report));
                break;
            case ts.DiagnosticCategory.Warning:
                this.logger.warn(__assign({ code: "TS" + diagnostic.code }, report));
                break;
            default:
                this.logger.info(__assign({ code: "TS" + diagnostic.code }, report));
                break;
        }
    };
    BaseTscRunner.prototype._reportDiagnostic = function (diagnostic) {
        var message = ts.flattenDiagnosticMessageText(diagnostic.messageText, "\n");
        if (diagnostic.file) {
            var _a = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start), line = _a.line, character = _a.character;
            this._reportSwitch(diagnostic, {
                fileName: diagnostic.file.fileName,
                line: line,
                character: character,
                message: message
            });
        }
        else {
            this._reportSwitch(diagnostic, {
                message: message
            });
        }
    };
    return BaseTscRunner;
}());
var BatchTscRunner = /** @class */ (function (_super) {
    __extends(BatchTscRunner, _super);
    function BatchTscRunner(options) {
        var _this = _super.call(this, __assign({}, options, { logger: options.logger.pushScope('BatchTscRunner') })) || this;
        _this._reportDiagnostics = _this._reportDiagnostics.bind(_this);
        return _this;
    }
    BatchTscRunner.prototype._batchCompile = function () {
        var _this = this;
        return this.finder.resolveFilePaths()
            .then(function (tsFiles) {
            var program = ts.createProgram(tsFiles, _this.config.compilerOptions);
            var emitResult = program.emit();
            return ts
                .getPreEmitDiagnostics(program)
                .concat(emitResult.diagnostics);
        })
            .then(this._reportDiagnostics)
            .catch(function (e) {
            _this.logger.error({
                scope: "_batchCompile",
                error: e
            });
        });
    };
    BatchTscRunner.prototype._batchJsAndDts = function () {
        var _this = this;
        if (this.config.rootPath !== this.config.outDir) {
            return this.finder.resolveJsWatcherFilePaths()
                .then(function (declPaths) {
                return Promise.map(declPaths, function (fromPath) {
                    return watcher.copyFile(_this.config, fromPath, _this.logger)
                        .catch(function (e) {
                        _this.logger.error({
                            scope: "_batchJsAndDts",
                            error: e
                        });
                    });
                })
                    .then(function () { });
            });
        }
        else {
            return Promise.resolve();
        }
    };
    BatchTscRunner.prototype.run = function () {
        var _this = this;
        return this._batchCompile()
            .then(function () { return _this._batchJsAndDts(); });
    };
    BatchTscRunner.prototype._reportDiagnostics = function (all) {
        var _this = this;
        all.forEach(function (diag) { return _this._reportDiagnostic(diag); });
    };
    return BatchTscRunner;
}(BaseTscRunner));
var WatchTscRunner = /** @class */ (function (_super) {
    __extends(WatchTscRunner, _super);
    function WatchTscRunner(options) {
        var _this = _super.call(this, options) || this;
        _this._formatHost = _this._getFormatHost();
        _this._reportWatchStatusChanged = _this._reportWatchStatusChanged.bind(_this);
        _this._jsWatcher = new watcher.JsWatcher(options);
        return _this;
    }
    WatchTscRunner.prototype.run = function () {
        this._createProgram = ts.createEmitAndSemanticDiagnosticsBuilderProgram;
        this._watchHost = ts.createWatchCompilerHost(this.config.configFilePath, undefined, ts.sys, this._createProgram, this._reportDiagnostic, this._reportWatchStatusChanged);
        this._watchProgram = ts.createWatchProgram(this._watchHost);
        return this._jsWatcher.run();
    };
    WatchTscRunner.prototype._reportWatchStatusChanged = function (diagnostic) {
        this._reportDiagnostic(diagnostic);
    };
    WatchTscRunner.prototype._getFormatHost = function () {
        var _this = this;
        return {
            getCanonicalFileName: function (path) {
                _this.logger.info({
                    getCanonicalFileName: path
                });
                return path;
            },
            getCurrentDirectory: ts.sys.getCurrentDirectory,
            getNewLine: function () { return ts.sys.newLine; }
        };
    };
    return WatchTscRunner;
}(BaseTscRunner));
function run(options) {
    return tsConfig.loadConfig()
        .then(function (config) {
        var runner = _createRunner({
            watch: options.watch || false,
            config: config,
            logger: options.logger
        });
        return runner.run();
    })
        .catch(function (error) {
        options.logger.error({
            scope: "run",
            error: error
        });
    });
}
exports.run = run;
function _createRunner(options) {
    if (options.watch) {
        return new WatchTscRunner(options);
    }
    else {
        return new BatchTscRunner(options);
    }
}
//# sourceMappingURL=tsc.js.map