import * as vpath from './vpath';
/**
 * TypeScript uses its own glob that differs from `glob`'s spec
 * and the `glob` used in `chokidar`, so we need something to
 * hold the logic that transforms between these 3 systems.
 *
 * Given that in our environment TypeScript will be the source,
 * we will start here.
 *
 * https://www.typescriptlang.org/docs/handbook/tsconfig-json.html
 *
 * `*` matches zero or more characters (excluding directory separators)
 * `?` matches any one character (excluding directory separators)
 * `**\/` (remove the backslash) recursively matches any sub directory
 *
 * if a segment of a glob pattern includes only `*` or `.*`, only files
 * with supported extensions are included (`*.ts`, `*.d.ts`, `*.tsx`.
 * if `allowJs` is set to true, then `*.js` and `*.jsx`).
 */
export interface TsConfigGlobOptions {
    spec: string;
    basePath?: string;
    allowJs?: boolean;
    allowTypes?: string[];
}
export declare class TsConfigGlob {
    readonly spec: vpath.PathObject;
    readonly basePath: vpath.PathObject;
    readonly allowTypes: string[];
    constructor(options: TsConfigGlobOptions);
    toFullPath(): vpath.PathObject;
    toRegExp(useFullPath?: boolean): RegExp;
    match(filePath: string): boolean;
    isRecursiveSpec(): boolean;
    isFileSpec(): boolean;
    hasExtension(extname: string): boolean;
    hasWildCard(): boolean;
    isDirectorySpec(): boolean;
    isWildCardSpec(seg?: string): boolean;
    toIncludeGlob(useFullPath?: boolean): string;
    toExcludeGlob(useFullPath?: boolean): string;
    toJsWatcherDirPath(useFullPath?: boolean): string;
    private _toExcludeGlob;
    private _stripWildCard;
    private _toIncludeGlob;
    globAllowTypeString(): string;
    private _useFullPath;
    private _segToRegEx;
}
