import * as Promise from 'bluebird';
import * as ts from 'typescript';
export interface TsConfigOptions {
    basePath: string;
    configName: string;
    compilerOptions: ts.CompilerOptions;
    files?: string[];
    include?: string[];
    exclude?: string[];
    extends?: string;
    compileOnSave?: boolean;
    prevConfig?: TsConfig;
}
export interface IncludedFileSpec {
    rootPath: string;
    include: string[];
    exclude: string[];
}
export declare class TsConfig {
    readonly basePath: string;
    readonly configName: string;
    private readonly _compilerOptions;
    private readonly _include?;
    private readonly _files?;
    private readonly _exclude?;
    private readonly _prevConfig?;
    constructor(options: TsConfigOptions);
    readonly configFilePath: string;
    readonly compilerOptions: ts.CompilerOptions;
    readonly rootDir: string;
    readonly isOutDir: boolean;
    readonly outDir: string;
    readonly excluded: string[];
    excludedDirs(): string[];
    private _getBaseExcluded;
    readonly files: string[] | undefined;
    readonly include: string[] | undefined;
    defaultExclude(): string[];
    defaultInclude(): string[];
    readonly rootPath: string;
    readonly outPath: string;
    toOutPath(filePath: string): string;
    toOutPaths(filePaths: string[]): string[];
    toRootPath(filePath: string): string;
    isIgnoredPath(filePath: string): boolean;
    /**
     * Moving a particular module from rootDir to outDir, and then rewrite
     * its path if it has outDir as part of the original path.
     *
     * i.e.
     *
     * config.moveModuleSpec('./lib/tsc.js', '../dist/lib/index') ==> '../lib/index'
     *
     * modulePath = './bin/tsc.js'
     * spec = '../dist/lib/index'
     *
     * targetModulePath = './dist/bin/tsc.js'
     * normalizedSpec = './dist/lib/index'
     * targetSpec = '../lib/index'
     *
     * @param modulePath - the relative path from the perspective of tsconfig.
     * @param spec - the spec inside of the module.
     */
    moveModuleSpec(modulePath: string, spec: string): string;
}
export declare function loadConfig(configName?: string, basePath?: string, recursive?: boolean): Promise<TsConfig>;
export declare function readConfigurationFile(resolvedConfig: ResolveConfigPathResult): TsConfigOptions;
interface ResolveConfigPathResult {
    basePath: string;
    configName: string;
    configData: string;
}
export declare function resolveConfigPath(configName: string, basePath: string, recursive?: boolean): Promise<ResolveConfigPathResult>;
export {};
