"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra-promise");
var ts = require("typescript");
var path = require("path");
var minimatch = require("minimatch");
var vPath = require("./vpath");
var _libMap = {
    es6: 'es2015',
    es7: 'es2016'
};
function _mapLibName(lib) {
    if (_libMap.hasOwnProperty(lib)) {
        return _libMap[lib];
    }
    else {
        return lib;
    }
}
function convertLibs(libs) {
    return libs.map(function (lib) { return "lib." + _mapLibName(lib) + ".d.ts"; });
}
function convertTarget(target) {
    switch (target.toLowerCase()) {
        case 'es3':
            return ts.ScriptTarget.ES3;
        case 'es5':
            return ts.ScriptTarget.ES5;
        case 'es6':
        case 'es2015':
            return ts.ScriptTarget.ES2015;
        case 'es7':
        case 'es2016':
            return ts.ScriptTarget.ES2016;
        case 'es2017':
            return ts.ScriptTarget.ES2017;
        case 'es2018':
            return ts.ScriptTarget.ES2018;
        // case 'es2019': // not in 3.0.3 (in 3.4.5)
        //     return ts.ScriptTarget.ES2019;
        case 'esnext':
            return ts.ScriptTarget.ESNext;
        default:
            return ts.ScriptTarget.Latest;
    }
}
function convertJsx(jsx) {
    switch (jsx) {
        case 'preserve':
            return ts.JsxEmit.Preserve;
        case 'react':
            return ts.JsxEmit.React;
        case 'react-native':
            return ts.JsxEmit.ReactNative;
        default:
            return ts.JsxEmit.None;
    }
}
function jsonToTsCompilerOptions(json) {
    var transformMap = {
        jsx: convertJsx,
        lib: convertLibs,
        target: convertTarget,
    };
    var result = Object.keys(json).reduce(function (acc, key) {
        if (transformMap.hasOwnProperty(key)) {
            acc[key] = transformMap[key](json[key]);
        }
        else {
            acc[key] = json[key];
        }
        return acc;
    }, {});
    if (!result.rootDir) {
        result.rootDir = '.';
    }
    if (!result.outDir) {
        result.outDir = '.';
    }
    return result;
}
// this isn't how it works, I think.
var TsConfig = /** @class */ (function () {
    function TsConfig(options) {
        this.basePath = options.basePath;
        this.configName = options.configName;
        this._compilerOptions = options.compilerOptions;
        this._include = options.include;
        this._exclude = options.exclude;
        this._prevConfig = options.prevConfig;
    }
    Object.defineProperty(TsConfig.prototype, "configFilePath", {
        get: function () {
            return path.join(this.basePath, this.configName);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TsConfig.prototype, "compilerOptions", {
        get: function () {
            if (this._prevConfig) {
                return Object.assign({}, this._prevConfig.compilerOptions, this._compilerOptions);
            }
            else {
                return this._compilerOptions;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TsConfig.prototype, "rootDir", {
        get: function () {
            return path.join(this.basePath, this.compilerOptions.rootDir || '.');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TsConfig.prototype, "isOutDir", {
        get: function () {
            return !!this.compilerOptions.outDir;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TsConfig.prototype, "outDir", {
        get: function () {
            return path.join(this.basePath, this.compilerOptions.outDir || '.');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TsConfig.prototype, "excluded", {
        // this excluded is not useable by our find function.
        // we want our find functions to work correctly with the globs here.
        get: function () {
            // outDir is already fullPath.
            var outDirs = this.outDir === this.rootDir ? [] : [this.compilerOptions.outDir || '.'];
            return outDirs.concat(this._getBaseExcluded())
                .filter(function (p) { return typeof (p) === 'string'; });
            // .map((spec) => this._normalizeGlob(spec))
        },
        enumerable: true,
        configurable: true
    });
    // excludedGlob() : string[] {
    //     return this.excluded.map((spec) => this._normalizeGlob(spec))
    // }
    TsConfig.prototype.excludedDirs = function () {
        var _this = this;
        var baseExcluded = this._getBaseExcluded();
        var outDirs = this.outDir === this.rootDir ? [] : [this.outDir];
        var excluded = baseExcluded.map(function (exclude) {
            if (exclude.endsWith(path.join('/**/*'))) {
                return exclude.substring(0, exclude.length - 5);
            }
            else {
                return exclude;
            }
        })
            .map(function (exclude) { return path.join(_this.basePath, exclude); });
        return outDirs.concat(excluded);
    };
    // exclude returns the "exclude" field, without full path resolution.
    TsConfig.prototype._getBaseExcluded = function () {
        if (this._exclude)
            return this._exclude;
        else if (this._prevConfig)
            return this._prevConfig.excluded;
        else
            return [
                'node_modules',
                'bower_components',
                'jspm_packages',
                '.git'
            ];
    };
    Object.defineProperty(TsConfig.prototype, "files", {
        // get excludedRegExp() : RegExp[] {
        //     return this.excluded.map((exclude) => minimatch.makeRe(path.join(this.basePath, exclude)))
        // }
        get: function () {
            if (this._files)
                return this._files;
            else if (this._prevConfig)
                return this._prevConfig.files;
            else
                return undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TsConfig.prototype, "include", {
        get: function () {
            if (this._include)
                return this._include;
            else if (this._prevConfig)
                return this._prevConfig.include;
            else
                return undefined;
        },
        enumerable: true,
        configurable: true
    });
    TsConfig.prototype.defaultExclude = function () {
        return [
            '**/*.d.ts',
        ];
    };
    TsConfig.prototype.defaultInclude = function () {
        var extensions = [
            'ts', 'tsx'
        ].concat(this.compilerOptions.allowJs ? ['.js', '.jsx'] : []);
        return ["**/*.{" + extensions.join(',') + "}"];
    };
    Object.defineProperty(TsConfig.prototype, "rootPath", {
        get: function () {
            return path.join(this.basePath, this.compilerOptions.rootDir || '.');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TsConfig.prototype, "outPath", {
        get: function () {
            return path.join(this.basePath, this.compilerOptions.outDir || '.');
        },
        enumerable: true,
        configurable: true
    });
    TsConfig.prototype.toOutPath = function (filePath) {
        var relFilePath = path.relative(this.rootPath, filePath);
        return path.join(this.outPath, relFilePath);
    };
    TsConfig.prototype.toOutPaths = function (filePaths) {
        var _this = this;
        return filePaths.map(function (filePath) { return _this.toOutPath(filePath); });
    };
    TsConfig.prototype.toRootPath = function (filePath) {
        var relFilePath = path.relative(this.outPath, filePath);
        return path.join(this.rootPath, relFilePath);
    };
    TsConfig.prototype.isIgnoredPath = function (filePath) {
        var excluded = this.excludedDirs();
        for (var i = 0; i < excluded.length; ++i) {
            if (minimatch(filePath, excluded[i]))
                return true;
        }
        return false;
    };
    /**
     * Moving a particular module from rootDir to outDir, and then rewrite
     * its path if it has outDir as part of the original path.
     *
     * i.e.
     *
     * config.moveModuleSpec('./lib/tsc.js', '../dist/lib/index') ==> '../lib/index'
     *
     * modulePath = './bin/tsc.js'
     * spec = '../dist/lib/index'
     *
     * targetModulePath = './dist/bin/tsc.js'
     * normalizedSpec = './dist/lib/index'
     * targetSpec = '../lib/index'
     *
     * @param modulePath - the relative path from the perspective of tsconfig.
     * @param spec - the spec inside of the module.
     */
    TsConfig.prototype.moveModuleSpec = function (modulePath, spec) {
        var fullModulePath = vPath.isAbsolute(modulePath) ? modulePath : path.join(this.rootPath, modulePath);
        var targetModulePath = this.toOutPath(fullModulePath);
        var normalizedSpec = path.join(path.dirname(fullModulePath), spec); // from rootPath
        if (normalizedSpec.startsWith(this.outDir)) {
            var targetSpec = path.relative(path.dirname(targetModulePath), normalizedSpec);
            return new vPath.PathObject(targetSpec).toVirtualPath();
        }
        else {
            return spec;
        }
    };
    return TsConfig;
}());
exports.TsConfig = TsConfig;
function loadConfig(configName, basePath, recursive) {
    if (configName === void 0) { configName = 'tsconfig.json'; }
    if (basePath === void 0) { basePath = process.cwd(); }
    if (recursive === void 0) { recursive = true; }
    return resolveConfigPath(configName, basePath, recursive)
        .then(function (result) {
        var configOptions = readConfigurationFile(result);
        if (configOptions.extends) {
            return loadConfig(configOptions.extends, result.basePath, false)
                .then(function (prevConfig) {
                return new TsConfig(__assign({}, configOptions, { basePath: result.basePath, configName: result.configName, prevConfig: prevConfig }));
            });
        }
        else {
            return new TsConfig(__assign({}, configOptions, { basePath: result.basePath, configName: result.configName }));
        }
    });
}
exports.loadConfig = loadConfig;
function readConfigurationFile(resolvedConfig) {
    var res = ts.readConfigFile(resolvedConfig.configName, function (filePath) { return resolvedConfig.configData; });
    var compilerOptions = jsonToTsCompilerOptions(res.config.compilerOptions);
    return __assign({}, res.config, { compilerOptions: compilerOptions });
}
exports.readConfigurationFile = readConfigurationFile;
function resolveConfigPath(configName, basePath, recursive) {
    if (recursive === void 0) { recursive = true; }
    var fullPath = path.join(basePath, configName);
    return fs.readFileAsync(fullPath, 'utf8')
        .then(function (configData) {
        return {
            basePath: path.dirname(fullPath),
            configName: path.basename(fullPath),
            configData: configData
        };
    })
        .catch(function (e) {
        if (e.code === 'ENOENT') {
            // what do we want to do?
            // determine that the basePath is no longer the rootPath.
            if (isRootPath(basePath) || !recursive) {
                throw e;
            }
            else {
                return resolveConfigPath(configName, path.dirname(basePath));
            }
        }
        else {
            throw e;
        }
    });
}
exports.resolveConfigPath = resolveConfigPath;
function isRootPath(filePath) {
    return path.dirname(filePath) === filePath;
}
//# sourceMappingURL=tsconfig.js.map