"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Promise = require("bluebird");
var glob = require("glob");
var fs = require("fs-extra-promise");
var path = require("path");
function copyFile(fromPath, toPath) {
    return fs.mkdirpAsync(path.dirname(toPath))
        .then(function () { return fs.copyAsync(fromPath, toPath); });
}
exports.copyFile = copyFile;
function readFile(filePath) {
    return fs.readFileAsync(filePath, 'utf8');
}
exports.readFile = readFile;
function writeFile(filePath, data) {
    return fs.mkdirpAsync(path.dirname(filePath))
        .then(function () { return fs.writeFileAsync(filePath, data, 'utf8'); });
}
exports.writeFile = writeFile;
function rmrf(filePath) {
    return fs.removeAsync(filePath);
}
exports.rmrf = rmrf;
function find(pattern, options) {
    var patterns = typeof (pattern) === 'string' ? [pattern] : pattern;
    return Promise.map(patterns, function (pattern) { return _findOne(pattern, options); })
        .then(function (results) { return _unique(results); });
}
exports.find = find;
function _unique(ars) {
    var map = new Map();
    return ars.reduce(function (acc, ary) {
        ary.forEach(function (item) {
            if (!map.has(item)) {
                map.set(item, true);
                acc.push(item);
            }
        });
        return acc;
    }, []);
}
function _findOne(pattern, options) {
    var globOptions = {
        cwd: options.cwd || process.cwd(),
        ignore: options.exclude || [],
        realpath: typeof (options.fullPath) === 'boolean' ? options.fullPath : false
    };
    return new Promise(function (resolve, reject) {
        glob(pattern, globOptions, function (err, matches) {
            if (err) {
                reject(err);
            }
            else {
                resolve(matches);
            }
        });
    });
}
//# sourceMappingURL=util.js.map