"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
exports.BASE_DELIM = '/';
exports.WINDOWS_DELIM = '\\';
// in linux / osx we are using /, in windows we can use / or use \.
// we want to make sure that they are correctly normalized..
var PathObject = /** @class */ (function () {
    function PathObject(filePath) {
        if (filePath === '') {
            throw new Error("EmptyFilePath");
        }
        this.filePath = path.normalize(filePath);
    }
    PathObject.prototype.dirname = function () {
        return new PathObject(path.dirname(this.filePath));
    };
    PathObject.prototype.basename = function (extname) {
        if (extname) {
            return path.basename(this.filePath, extname);
        }
        else {
            return path.basename(this.filePath);
        }
    };
    PathObject.prototype.extname = function () {
        return path.extname(this.filePath);
    };
    PathObject.prototype.join = function () {
        var segs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            segs[_i] = arguments[_i];
        }
        return new PathObject(path.join.apply(path, [this.filePath].concat(segs)));
    };
    PathObject.prototype.relative = function (toPath) {
        return new PathObject(path.join(this.filePath, toPath));
    };
    PathObject.prototype.resolve = function () {
        var pathSegments = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            pathSegments[_i] = arguments[_i];
        }
        return new PathObject(path.resolve.apply(path, [this.filePath].concat(pathSegments)));
    };
    PathObject.prototype.segments = function () {
        return this.filePath.split(path.sep);
    };
    PathObject.prototype.lastSeg = function () {
        var segments = this.segments();
        if (segments.length == 0)
            throw new Error("EmptyFilePath");
        return segments[segments.length - 1];
    };
    PathObject.prototype.toOsPath = function () {
        return this.filePath;
    };
    PathObject.prototype.toVirtualPath = function () {
        if (path.sep === exports.WINDOWS_DELIM) {
            return this.filePath.replace(/\\/g, exports.BASE_DELIM);
        }
        else {
            return this.filePath;
        }
    };
    PathObject.prototype.toString = function () {
        return this.toVirtualPath();
    };
    PathObject.prototype.hasDrive = function () {
        return this.filePath.match(/^[a-zA-Z]\:/);
    };
    PathObject.prototype.isAbsolute = function () {
        if (path.sep === exports.WINDOWS_DELIM && this.hasDrive()) {
            return true;
        }
        else {
            return this.filePath.startsWith(path.sep);
        }
    };
    PathObject.prototype.isRelative = function () {
        return !this.isAbsolute();
    };
    return PathObject;
}());
exports.PathObject = PathObject;
function isAbsolute(filePath) {
    return new PathObject(filePath).isAbsolute();
}
exports.isAbsolute = isAbsolute;
function isRelative(filePath) {
    return new PathObject(filePath).isRelative();
}
exports.isRelative = isRelative;
function join(filePath) {
    var segs = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        segs[_i - 1] = arguments[_i];
    }
    var _a;
    return (_a = new PathObject(filePath)).join.apply(_a, segs);
}
exports.join = join;
function relative(fromPath, toPath) {
    return new PathObject(fromPath).relative(toPath);
}
exports.relative = relative;
function resolve(filePath) {
    var segs = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        segs[_i - 1] = arguments[_i];
    }
    var _a;
    return (_a = new PathObject(filePath)).resolve.apply(_a, segs);
}
exports.resolve = resolve;
function dirname(filePath) {
    return new PathObject(filePath).dirname();
}
exports.dirname = dirname;
function basename(filePath, extname) {
    return new PathObject(filePath).basename(extname);
}
exports.basename = basename;
function extname(filePath) {
    return new PathObject(filePath).extname();
}
exports.extname = extname;
//# sourceMappingURL=vpath.js.map